###  Generates Figure A12
library(QPdecon)
library(deamer)

cdf2quant = function(xx,cdf,pr)
{
  q = matrix(nrow=length(pr),ncol=1)
  for (i in 1:length(pr))
  {
    ind = (cdf>pr[i])
    q[i] = min(xx[ind])
  }
  q
}

set.seed(38538)
n <- 1000
shape = 1.5
rate = sqrt(shape)
qq = c(0.025,0.05,0.1,0.25,0.5,0.75,0.9,0.95,0.975)
dmodel = function(x) dgamma(x,shape=shape,rate=rate)
rmodel = function(n) {rgamma(n,shape=shape,rate=rate)}
Nsim = 100
quantResults = matrix(nrow=Nsim,ncol=3*length(qq))
pdfResults = matrix(nrow=Nsim,ncol=3)
n = 1000
sdz = 0.75
K = 200
f_Z = sdz
noise.type = "Gaussian"
TRUEquant = qgamma(qq,shape=shape,rate=rate)
truePDF = dgamma(TRUEquant,shape,rate) 
QPquant = matrix(nrow=Nsim,ncol=length(TRUEquant))
QPpdf = QPquant
PCquant = matrix(nrow=Nsim,ncol=length(TRUEquant))
PCpdf = QPquant
for (isim in 1:Nsim)
{
  x = rgamma(n,shape,rate)
  z = rnorm(n,mean=0,sd=sdz)
  y = x + z
  fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=f_Z, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE,  
                  convex = NULL, support = c(0,max(y)))
  fitPC = deamerKE(y,sigma=sdz,grid.length=K,mu=0,noise.type=noise.type)
  PCcdf = cumsum(fitPC$f); PCcdf = PCcdf/PCcdf[K-1]
  PCquant = cdf2quant(fitPC$supp,PCcdf,qq) 
  QPcdf = cumsum(fitQP$f_X); QPcdf = QPcdf/QPcdf[length(QPcdf)]
  QPquant[isim,] = cdf2quant(fitQP$x,QPcdf,qq)

  QPpdf[isim,] = spline(y=fitQP$f_X[(fitQP$x>0)],x=fitQP$x[(fitQP$x>0)],xout=TRUEquant)$y
  PCpdf[isim,] = spline(y=fitPC$f,x=fitPC$supp,xout=TRUEquant)$y
  print(isim)
}
pdf("biasSDgamma1dot5.pdf",width=7,height=3.4)
par(mfrow=c(1,2))
QPbiasPDF = colMeans(QPpdf) - truePDF
plot(qq,QPbiasPDF,type="b",main="QP",ylim=c(-0.2,0.12),
     xlab="probability",ylab="bias and SD",lwd=2,cex=1.25)
lines(qq,apply(QPpdf,2,sd),col="red",type="b",lty=4,pch="*",lwd=3,cex=1.5)
abline(h=0)
legend("bottomright",c("bias","SD"),col=c("black","red"),lty=c(1,4),pch=c("o","*"),lwd=3)

PCbiasPDF = colMeans(PCpdf) - truePDF
plot(qq,PCbiasPDF,type="b",main="PC",ylim=c(-0.2,0.12),
     xlab="probability",ylab="bias and SD",lwd=2,cex=1.25)
lines(qq,apply(PCpdf,2,sd),col="red",type="b",lty=4,pch="*",lwd=3,cex=1.5)
abline(h=0)
legend("bottomright",c("bias","SD"),col=c("black","red"),lty=c(1,4),pch=c("o","*"),lwd=3)
graphics.off()

save.image("BiasSDgamma1dot5.RData")
