#' @export
SUREPlot <-
function(Y, Pdf=TRUE, K=NULL, f_Z, lambda=NULL, reg="2Deriv", integr=TRUE, nonneg=TRUE, monotone=NULL, convex=NULL, support=NULL) {

######################################################################################
##The folowing block of code was copied directly from QPdecon()
	N=length(Y)
	if(is.null(K)) K = min(200, floor(3 * sqrt(N)))

	#Call find.CDd() to compute the following information, to be used for computing various quantities in the QP optimization
	CDd <- find.CDd(Y=Y, Pdf=Pdf, K=K, f_Z=f_Z, reg=reg, support=support) 
	x <- CDd$x  #K-length vector of x values over which f_X is estimated
	x0 <- CDd$x0 #K_sup-length vector of x values for only the specified support of X. All elements of x0 are a subset of x
	delta <- CDd$delta #x discretization interval
	f_Y <- CDd$f_Y #Estimated density of Y, from its histogram
	C <- CDd$C #KxK_sup Convolution matrix C
	K_sup <- CDd$K_sup #number of discrete x values over the support
	K <- CDd$K #number of discrete x values over the entire range
	l_sup <- CDd$l_sup #lower boundary of support
	u_sup <- CDd$u_sup #upper boundary of support
	CtC <- CDd$CtC  #t(C)%*%C
	CtfY <- CDd$CtfY #t(C)%*%f_Y
	CtSig <- CDd$CtSig #t(C)%*%Sigma, where Sigma is the covariance matrix of f_Y
	DtD <- CDd$DtD  #the K_supxK_sup matrix in the quadratic regularization (I, t(D2)%*%D2, etc) 
	f_reg <- CDd$f_reg #density to which f_X is regularized. Zero-vector if reg == "2Deriv"

	#Call find.Ab() to compute the following constraint information, to be used in the QP optimization
	Ab <- find.Ab(Pdf=Pdf, K_sup=K_sup, integr=integr, nonneg=nonneg, monotone=monotone, convex=convex, l_sup=l_sup, u_sup=u_sup, x0=x0)
	meq <- Ab$meq   ##the number of equality constraints (1 or 0, the former if integr==TRUE
	A <- Ab$A  ##A consraint matrix to be used in solve.QP
	b <- Ab$b  ##b constraint vector to be used in solve.QP
##End of block copied from QPdecon()
######################################################################################

	#If lambda is NULL, find the SURE estimate of lambda, and create the vector of lambda values to be centered in log spacing around the SURE value 
	if (is.null(lambda)) {
		opt.fun <- function(mu, ...) {out = f_XandSURE(mu, ...); out$SURE.value} ##create wrapper function for f_XandSURE for optimization
		OPT <- optimize(opt.fun, c(-20,5), f_Y=f_Y, Pdf=Pdf, integr=integr, D=NULL, d=NULL, C=C, CtC=CtC, DtD=DtD, CtfY=CtfY, CtSig=CtSig, f_reg=f_reg, A=A, b=b, meq=meq)
		lambda.SURE <- exp(OPT$minimum) #The SURE estimate of lambda
		lambda <- lambda.SURE*(2^c(-10:30)) 
	} else if (min(lambda)<0) {
		stop("all values of lambda must be nonnegative")
	}

	##For each value of lambda in the sequence, call f_XandSURE() to estimate f_X and compute the resulting SURE value
	SURE.value <- numeric(length(lambda)) ##SURE criterion value
	for (j in 1:length(lambda)) {
		OUT <- f_XandSURE(mu=log(lambda[j]), f_Y=f_Y, Pdf=Pdf, integr=integr, D=NULL, d=NULL, C=C, CtC=CtC, DtD=DtD, CtfY=CtfY, CtSig=CtSig, f_reg=f_reg, A=A, b=b, meq=meq)
		SURE.value[j] <- OUT$SURE.value
	} ##End of for (j in 1:length(lambda))

	##Plot the SURE value vs. lambda in log scale
	par(mfrow=c(1,1), family="serif")
	plot(lambda, SURE.value, type="l", xlab=expression(italic(lambda)), ylab=expression("SURE value"), log="xy")

	return(list(lambda.seq=lambda, SURE.value=SURE.value))

}