find.Ab <-
function(Pdf, K_sup, integr, nonneg, monotone, convex, l_sup, u_sup, x0) {

	A <- numeric(0)
	b <- numeric(0)
	meq <- 0
	delta <- x0[2]-x0[1]

	#If integr==TRUE, incorporate equality constraint that f_X integrates to 1.0
	if((integr==TRUE) && (Pdf==TRUE)){
		meq <- 1
		A <- cbind(A,matrix(delta,K_sup,1))
		b<- c(b,as.vector(c(1)))
	}else if((integr==TRUE) && (Pdf==FALSE)){
		meq <- 1
		A <- cbind(A, matrix(c(rep(0,(K_sup-1)), 1), K_sup, 1))
		b<- c(b, as.vector(c(1)))
	}##end of if (integr==TRUE) block	
	
	#Incorporate pdf nonnegativity constraints if nonneg == TRUE and Pdf == TRUE
	if ((nonneg == TRUE) && (Pdf == TRUE)) { 
		A<-cbind(A,diag(K_sup))
		b<-c(b,rep(0,K_sup))
	}

	#Incorporate constraint that 0 <= cdf <=1 if nonneg == FALSE and Pdf == FALSE
	if ((nonneg == FALSE) && (Pdf == FALSE)) { 
		A<-cbind(A,diag(K_sup))
		b<-c(b,rep(0,K_sup))
		A<-cbind(A,-diag(K_sup))
  		b<-c(b,rep(-1,K_sup))
	}

	#Incorporate pdf nonnegativity and 0 <= cdf <=1 constraints on cdf if nonneg == TRUE and Pdf == FALSE
	if ((nonneg == TRUE) && (Pdf == FALSE)) { 
		A<-cbind(A,diag(K_sup))
		b<-c(b,rep(0,K_sup))
		A<-cbind(A,-diag(K_sup))
  		b<-c(b,rep(-1,K_sup))
		for (i in 1:(K_sup-1)) {
		      a<-rep(0,K_sup)
	      	a[i] <- -1
		      a[i+1] <- 1
		      A<-cbind(A,a)
		      b<-c(b,0)
		}##end for
	}

	###Incorporate monotonicity and convex constraints on the pdf if present. Ignore if Pdf == FALSE
	if(Pdf==TRUE){
		if (!is.null(monotone)) { #incorporate monotonicity constraints
	
			l_mon <- monotone[1]
			u_mon <- monotone[2]
			if(!is.na(l_mon)){
				if(l_mon<l_sup) stop("monotone constraint invalid, exceeds lower bound!")
				l_ind<-min(which(x0>=l_mon))
			    	for (i in 2:l_ind) {
					a<-rep(0,K_sup)
	      			a[i] <- 1
	      			a[i-1] <- -1
	     				A<-cbind(A,a)
	      			b<-c(b,0)
	   			 }## end for
			}##end if(c(num,NA) or c(num,num)) case

			if(!is.na(u_mon)){
				if(u_mon>u_sup) stop("monotone constraint invalid, exceeds upper bound!")
				u_ind<-max(which(x0<=u_mon))
	    			for (i in (u_ind+1):(K_sup-1)) {
	    				a<-rep(0,K_sup)
	      			a[i] <- 1
	      			a[i+1] <- -1
	      			A<-cbind(A,a)
	      			b<-c(b,0)
	    			}##end for
				
			}##end if(c(NA,num) or c(num,num)) case
		}##end of "if (!is.null(monotone))" expression

   		if (!is.null(convex)) { #Incorporate convexity constraints
  			l_con <- convex[1]
  			u_con <- convex[2]
  			if(!is.na(l_con)){
				if(l_con<l_sup) stop("convex constraint invalid, exceeds lower bound!")
				l_ind<-min(which(x0>=l_con))
    				for (i in 3:l_ind) {
      				a<-rep(0,K_sup)
      				a[i] <- 1
      				a[i-1] <- -2
      				a[i-2] <- 1
      				A<-cbind(A,a)
      				b<-c(b,0)
    				}##end for

			}##end of if(!is.na(l_con)) block

			if(!is.na(u_con)){
				if(u_con>u_sup) stop("convex constraint invalid, exceeds upper bound!")
				u_ind<-max(which(x0<=u_con))
    				for (i in (u_ind+2):K_sup-2) {
      				a<-rep(0,K_sup)
      				a[i] <- 1
      				a[i+1] <- -2
      				a[i+2] <- 1
      				A<-cbind(A,a)
      				b<-c(b,0)
    				}##end for
				
			}##end of if(!is.na(u_con)) block
		}  #end of "if (!is.null(convex))" block

	}##end monotone and convex constraints for if(Pdf==TRUE) block  



	if(!length(A)) {
		A=as.matrix(rep(0,K_sup),K_sup,1)
		b=0
	}

	Ab <- NULL
	Ab$meq <- meq   ##the number of equality constraints (1 or 0, the former if integr==TRUE
	Ab$A <- A  ##A consraint matrix to be used in solve.QP
	Ab$b <- b  ##b constraint vector to be used in solve.QP

	return(Ab)
}
