find.CDd <-
function(Y, Pdf, K, f_Z, reg, support){

	N <- length(Y)
	if(!typeof(f_Z)=="closure"){
		SDZ=f_Z	#standard deviation of noise Z
	}else{
		f_Z1<-function(z)	z*f_Z(z)
		f_Z2<-function(z) z^2*f_Z(z)
		EZ=integrate(f_Z1,-Inf,Inf)[[1]]
		if(abs(EZ)>1e-20) stop("noise distribution must be mean zero!")
		SDZ=sqrt(max(0,integrate(f_Z2,-Inf,Inf)[[1]])-0)
	}

	xmin=min(Y)
	xmax=max(Y)
	delta=(xmax-xmin)/(K-1) #may get reset later

	
	#Use support constraints, if present, to find x, x0, delta, K, K_sup, l_sup, and u_sup
	if (!is.null(support)) {
	  l_sup <- c(support[1])
	  u_sup <- c(support[2])
	  if(is.na(l_sup)&&is.na(u_sup)){
		stop("support constraint invalid, missing a lower and upper bound!")
	  }else if(!is.na(l_sup)&&is.na(u_sup)){
		u_sup=max(Y)
	  }else if(is.na(l_sup)&&!is.na(u_sup)){
		l_sup=min(Y)
	  }
	  if(u_sup==Inf) u_sup = max(Y)
	  if(l_sup==-Inf) l_sup = min(Y)
	  #discretize the support range of the pdfs
	  if(xmin>u_sup||xmax<l_sup){
		stop("support is not well defined!")
	  }else if(l_sup>xmin&&u_sup<xmax){
		delta=(u_sup-l_sup)/ceiling((u_sup-l_sup)/delta)
		x0=seq(from=l_sup, to=u_sup, by=delta)
		x<-c(seq(from=l_sup, to=xmin, by=-delta), x0, seq(from=u_sup, to=xmax, by=delta))
	  }else if(l_sup<=xmin&&u_sup>=xmax){
		delta=(xmax-xmin)/(K-1)
		x<-seq(from=xmin, to=xmax, length= K)
		x0=x
	  }else if(l_sup<xmin&&u_sup<xmax){
		delta=(u_sup-xmin)/ceiling((u_sup-xmin)/delta)
		x<-c(seq(from=u_sup, to=xmin, by=-delta), seq(from=u_sup, to=xmax, by=delta))
		x0=seq(from=xmin, to=u_sup, by=delta)
	  }else{
		delta=(xmax-l_sup)/ceiling((xmax-l_sup)/delta)
		x<-c(seq(from=l_sup, to=xmin, by=-delta), seq(from=l_sup, to=xmax, by=delta))
		x0=seq(from=l_sup, to=xmax, by=delta)	
	 }
	 x=sort(unique(x))
	 xx = c(x[1])
	 for( i in 2:length(x)) if(abs(x[i]-x[i-1])> delta*0.0001) xx = c(xx,x[i])
	 x = xx
	 K = length(x)
	 K_sup=length(x0)
	
	}else{
		l_sup <-xmin
	 	u_sup <-xmax
		K_sup=K
		x0=x=seq(from=xmin, to=xmax, length.out=K)
	
	} #end of "if (!is.null(support))" expression


	#calculate the estimated density f_Y of Y from the sample
	br=x-delta/2
	br=c(br,xmax+delta/2)
	if(x[1]-delta/2>xmin) br=c(xmin-delta/2,br)
	histo=hist(Y,breaks=br,plot=FALSE)
	if(Pdf==TRUE){
		f_Y <-histo$density
	  	if(x[1]-delta/2>xmin)	f_Y=f_Y[-1]
		f_Y<-f_Y/(sum(f_Y)*delta) #rescale so f_Y integrates to 1.0 
	}else{
		F_Y=ecdf(Y)
		f_Y=F_Y(x)
	}

	##Calculate the KxK_sup convolution matrix C
	C<-matrix(nrow=K,ncol=K_sup)

   	if(!typeof(f_Z)=="closure"){ #Compute C for Gaussian noise when f_Z is a standard deviation
		for (i in 1:K_sup)  C[,i]<- pnorm(x-x0[i]+delta/2, sd = SDZ) - pnorm(x-x0[i]-delta/2, sd = SDZ)	
	}else{ #Compute C for general f_Z
		
		f_Z.int<-rep(0, 2*K-1)
		lower <-(1-K-1/2)*delta
		for(k in 1:(2*K-1)){
			f_Z.int[k]<-integrate(f_Z, lower = lower, upper = lower+delta)[[1]]
			lower <-lower+delta
		}
		for (i in 1:K_sup){
			first.index<-round((x[1]-x0[i])/delta)+K
			C[,i]<-f_Z.int[first.index:(first.index+K-1)]
		}
	}

	#Compute various matrices, to be used later in the QP objective function or SURE computations
	CtC <- t(C)%*%C  ##Used in objective function
	CtfY <- t(C)%*%f_Y  ##Used in objective function
	CtSig <- t(C)%*%(diag(f_Y/(N*delta))-outer(f_Y/N,f_Y)) ##Used in SURE computations


	##Calculate the K_sup x K_sup matrix DtD and the K_sup x 1 vector f_reg. After specifying a lambda, 
	##these will be used to compute in the D matrix and d vector in the QP objective function
	if (reg == "Gauss") {
		VarY <- var(Y)
		MeanY <- mean(Y)
		SDX <- sqrt(max((delta/5)^2,VarY-SDZ^2))
		f_reg<- pnorm(x0+delta/2, mean=MeanY, sd = SDX) - pnorm(x0-delta/2, mean=MeanY, sd = SDX)	
		if (Pdf == FALSE) {
			f_reg <- cumsum(f_reg)
		}
		DtD <- diag(K_sup)
	}

	if (reg == "Unif") {
		f_reg<- rep(1/delta, K_sup)
		if (Pdf == FALSE) {
			f_reg <- cumsum(f_reg)
		}
		DtD <- diag(K_sup)
	}

	if(reg == "2Deriv"){
		f_reg=rep(0,K_sup)
		DtD = matrix(0,K_sup,K_sup)
		if (Pdf == TRUE) { #Compute DtD = t(D2)%*%D2, where D2 is the second finite difference matrix
			for (i in 3:(K_sup-2)) DtD[i,(i-2):(i+2)] <- DtD[i,(i-2):(i+2)] + c(1,-4,6,-4,1)/delta^4
			DtD[1,1:3] <- DtD[1,1:3] + c(1,-2,1)/delta^4
			DtD[2,1:4] <- DtD[2,1:4] + c(-2,5,-4,1)/delta^4
			DtD[K_sup-1,(K_sup-3):K_sup] <- DtD[K_sup-1,(K_sup-3):K_sup] + c(1,-4,5,-2)/delta^4
			DtD[K_sup,(K_sup-2):K_sup] <- DtD[K_sup,(K_sup-2):K_sup] + c(1,-2,1)/delta^4
		}
		else { #Compute DtD = t(D3)%*%D3, where D3 is the third finite difference matrix
			for (i in 4:(K_sup-3)) DtD[i,(i-3):(i+3)] <- DtD[i,(i-3):(i+3)] + c(-1,6,-15,20,-15,6,-1)/delta^6
			DtD[1,1:4] <- DtD[1,1:4] + c(1,-3,3,-1)/delta^6
			DtD[2,1:5] <- DtD[2,1:5] + c(-3,10,-12,6,-1)/delta^6
			DtD[3,1:6] <- DtD[3,1:6] + c(3,-12,19,-15,6,-1)/delta^6
			DtD[K_sup-2,(K_sup-5):K_sup] <- DtD[K_sup-2,(K_sup-5):K_sup] + c(-1,6,-15,19,-12,3)/delta^6
			DtD[K_sup-1,(K_sup-4):K_sup] <- DtD[K_sup-1,(K_sup-4):K_sup] + c(-1,6,-12,10,-3)/delta^6
			DtD[K_sup,(K_sup-3):K_sup] <- DtD[K_sup,(K_sup-3):K_sup] + c(-1,3,-3,1)/delta^6
		}
	}

	CDd <- NULL
	CDd$C <- C #KxK_sup Convolution matrix C
	CDd$CtC <- CtC  #t(C)%*%C
	CDd$CtfY <- CtfY #t(C)%*%f_Y
	CDd$CtSig <- CtSig #t(C)%*%Sigma, where Sigma is the covariance matrix of f_Y
	CDd$x <- x  #K-length vector of x values over which f_X is estimated
	CDd$x0 <- x0 #K_sup-length vector of x values for only the specified support of X. All elements of x0 are a subset of x
	CDd$f_Y <- f_Y #Estimated density of Y, from its histogram
	CDd$DtD <- DtD  #t(D)%*%D
	CDd$K_sup <- K_sup #number of discrete x values over the support
	CDd$K <- K #number of discrete x values over the entire range
	CDd$l_sup <- l_sup #lower boundary of support
	CDd$u_sup <- u_sup #upper boundary of support
	CDd$delta <- delta #x discretization interval
	CDd$f_reg <- f_reg #density (K_sup-length vector) to which f_X is regularized. Zero-vector if reg == "2Deriv"

	return(CDd)
}
