\name{SUREPlot}
\alias{SUREPlot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	SURE plot generation
}
\description{
This function returns a SURE plot to help determining the value of the regularization parameter \eqn{\lambda} in the QP objective function |f_Y-Cf_X|^2+\eqn{\lambda}Q(f_X) used in the QPdecon function. The SURE plot is a plot of the SURE values (in log base 10 scale) for the optimization solution, as a function of \eqn{\lambda}, and the best \eqn{\lambda} value should be chosen just to the right of the elbow point of the SURE plot.
}
\usage{
SUREPlot(Y, Pdf = TRUE, K = NULL, f_Z, lambda = NULL, reg = "2Deriv", integr = TRUE, nonneg = TRUE, monotone = NULL, convex = NULL, support = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y}{
	N-length vector of N observations of Y
}
  \item{Pdf}{
	set \code{Pdf=TRUE} to obtain the pdf estimator, or set \code{Pdf=FALSE} to obtain the cdf estimator   
}
  \item{K}{
	number of discrete points at which the pdfs will be evaluated over the range of X and Y

}
  \item{f_Z}{
	the pdf of the noise Z. For zero-mean Gaussian Z, \code{f_Z} is a constant that represents the standard deviation of Z. For any other noise distribution, \code{f_Z} should be a user-defined function that takes a scalar (say \eqn{z}) as an input argument and returns a scalar value for the noise density f_Z(\eqn{z}) at \eqn{Z = z}

}
  \item{lambda}{
	To use a specific value for the regularization parameter, set \code{lambda} equal to the value. Otherwise, to have \code{lambda} chosen automatically (by the SURE method), set \code{lambda = NULL}

}
  \item{reg}{
	regularization method can be set as \code{reg = "Gauss", "2Deriv"} or \code{"Unif"}. \code{reg="Gauss"} sets the regularization to be |f_X-f_reg|^2, where f_reg is a Gaussian distribution having the same mean and variance as X (which is determined internally, based on the sample variance of Y and the noise variance from \code{f_Z}); \code{reg = "2Deriv"}, which is the default, penalizes the magnitude of the second-derivative of \code{f_X};  \code{reg="Unif"} sets f_reg as a uniform distribution over the range of Y

}
  \item{integr}{
	if \code{Pdf=TRUE}, setting \code{integr=TRUE} enforces the integrate-to-one constraint on \code{f_X}; if \code{Pdf=FALSE}, i.e. to obtain cdf estimator, then setting \code{integr=TRUE} enforces the last element in the vector \code{f_X} to be 1
}
  \item{nonneg}{
	set \code{nonneg=TRUE} to include the nonnegativity constraint that each element of \code{f_X} is nonnegative
}
  \item{monotone}{
	set as a two-length vector \code{monotone=c(l_mon,u_mon)} to specify the monotonic regions. \code{f_X} is constrained to be monotonically increasing for \code{x} below \code{l_mon} and decreasing above \code{u_mon}. If \code{monotone=NULL}, no monotonicity constraints will be used. For only upper-tail monotonicity, specify \code{l_mon} as \code{NA}, and let \code{u_mon} equal the value after which the pdf is monotonically decreasing. Likewise, for lower-tail monotonicity, let \code{l_mon} equal the value below which the pdf is monotonically increassing, and set \code{u_mon} as \code{NA}  
}
  \item{convex}{
	set as a two-length vector \code{convex=c(l_con,u_con)} to specify the convex regions. \code{f_X} is constrained to be convex for \code{x} below \code{l_con} and above \code{u_con}. If \code{convex=NULL}, no convexity constraints will be used. For only upper-tail convexity, specify \code{l_con} as \code{NA}, and let \code{u_con} equal the value after which the pdf is convex. Likewise, for lower-tail convexitity, let \code{l_con} equal the value below which the pdf is convex, and set \code{u_con} as \code{NA}
}
  \item{support}{
	set as a two-length vector \code{support=c(l_sup,u_sup)} to specify the lower and upper end of the support; the default is \code{support=c(min(Y),max(Y))}; the lower and upper end can be specified to be \code{-Inf} or \code{Inf}, respectively  
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
Return a SURE plot for regularization parameter check and:
\item{lambda.seq }{a vector of \eqn{\lambda} values on the horizontal axis of the SURE plot, automatically determined by the \code{SUREPlot} function or specified by users}
\item{SURE.value}{a vector of the SURE values corresponding to \code{lambda.seq} (on the vertical axis of the SURE plot)}
%% ...
}
\references{
Ran Yang, Daniel Apley, Jeremy Staum and David Ruppert. Density Deconvolution with Additive Measurement Errors using Quadratic Programming.
}
\author{
Ran Yang, D. W. Apley
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
rm(list=ls())

X=rgamma(3000,5,1)
Z=rnorm(3000,0,sd=1.8)
Y=X+Z
f_Z<-function(z){dnorm(z,mean=0,sd=1.8)}
K=min(200,floor(3*sqrt(3000)))

###SURE plots####
S=SUREPlot(Y=Y,Pdf=TRUE,K=K,f_Z=f_Z,lambda=NULL,
reg="2Deriv",integr=TRUE, nonneg=TRUE)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
