###  Generates data for Table 1, row 2
yesPlot = TRUE
yesSave = TRUE
library(BayesME)
library(QPdecon)
library(mixtools)
source("myBayes.R")
set.seed(38538)
n <- 1000
shape = 5
rate = sqrt(shape)
dmodel = function(x) dgamma(x,shape=shape,rate=rate)
rmodel = function(n) {rgamma(n,shape=shape,rate=rate)}
nsim = 100
MAE_QP = matrix(nrow =  nsim, ncol = 1)
MAE_Bayes = MAE_QP
timeQP = MAE_QP
timeBayes = MAE_QP
lambda = MAE_QP
for (isim in 1:nsim)
{
  x <- rmodel(n)
  u <- rnorm(2*n,sd=1)
  w <- kronecker(x,rep(1,2)) + u
  #w <- c(x,x) + u
  ind <- kronecker(1:n,rep(1,2))
  t1 = proc.time()
  fitBayes = BayesDecon(w,ind,burnin=500,simsize=1000,error.fit="normal")
  t2 = proc.time()
  timeBayes[isim] = (t2-t1)[3]
  g = seq(1,(2*n-1),by=2)
  y = (w[g] + w[1+g])/2
  f_Z = mean((w[g]-w[g+1])^2)/4
  f_Z = sqrt(f_Z)
  t3 = proc.time()
  fitQP = QPdecon(y,f_Z=f_Z,support = c(0,max(y)))
  t4 = proc.time()
  lambda[isim] = fitQP$lambda
  timeQP[isim] = (t4-t3)[3]
  xQP = fitQP$x
  xBayes = fitBayes$interval
  trueDenQP = dmodel(xQP)
  trueDenBayes =dmodel(xBayes)
  MAE_QP[isim] = sum(abs(fitQP$f_X - trueDenQP) ) * (xQP[2]-xQP[1])
  MAE_Bayes[isim] = sum(abs(fitBayes$density - trueDenBayes) ) * (xBayes[2]-xBayes[1])
  plot(fitQP$x,fitQP$f_X,type="l",col="red",xlab="x",ylab="density",ylim=c(0,0.7))
  lines(xQP,trueDenQP)
  lines(fitBayes$interval,fitBayes$density,col="blue")
  legend("topright",c("true","Bayes","QP"),lty=1,col=c("black","blue","red"))
  print(isim)
}


result = c(mean(MAE_Bayes),
           mean(MAE_QP),
           mean(MAE_Bayes)/mean(MAE_QP),
           mean(timeBayes),
           mean(timeQP))
round(result,3)

result2 = c(median(MAE_Bayes),
            median(MAE_QP),
            median(MAE_Bayes)/median(MAE_QP),
            median(timeBayes),
            median(timeQP))
round(result2,3)

if (yesPlot == TRUE) pdf("lambdaMAE-gamma5.pdf",width=6,height=5)
plot(lambda,MAE_QP)
if (yesPlot == TRUE)  graphics.off()


if (yesSave == TRUE) save.image("QPversusBayesGamma5.RData")

###  load("QPversusBayesGamma5.RData")
