###  Generates Figure 9
###  Compares KD and PC to QP using medians

##  Specify the model
shape = 1.5
rate = sqrt(shape)
qq = c(0.05,0.1,0.25,0.5,0.75,0.9,0.95)
TRUEquant = qgamma(qq,shape=shape,rate=rate)
rmodel = function(n) {rgamma(n,shape=shape,rate=rate)}
den = "gamma"
unimodal = TRUE
useLaplace = FALSE
monotone = FALSE
K = 200
variableK = FALSE
##  End model specification
Nrep = 200
t1 = proc.time()
source("runSim.R")
t2 = proc.time()
(t2-t1)/60

widthMult = 0.95
width = 8*widthMult
height = 6*widthMult
cex.label = 1.35
pdf("gamma1dot5GaussMedian.pdf",width=width,height=height)
par(mfrow=c(2,2))
xlimL = -750
legendCex = 1.1
ylab="log ratio of MedAEs"
## KD versus QP for quantiles
plot(sampSize,tabKD_QP_quant[,4],type="b",ylim=c(min(tabKD_QP_quant),
                                                 max(tabKD_QP_quant)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Gamma(1.5): KD - QP quantiles",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabKD_QP_quant[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabKD_QP_quant[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabKD_QP_quant[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## KD versus QP for pdf's
plot(sampSize,tabKD_QP_pdf[,4],type="b",ylim=c(min(tabKD_QP_pdf),max(tabKD_QP_pdf)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Gamma(1.5): KD - QP pdf's",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabKD_QP_pdf[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabKD_QP_pdf[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabKD_QP_pdf[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## PC versus QP for quantiles
plot(sampSize,tabPC_QP_quant[,4],type="b",ylim=c(min(tabPC_QP_quant),
                                                 max(tabPC_QP_quant)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Gamma(1.5): PC - QP quantiles",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabPC_QP_quant[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabPC_QP_quant[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabPC_QP_quant[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## PC versus QP for pdf's
plot(sampSize,tabPC_QP_pdf[,4],type="b",ylim=c(min(tabPC_QP_pdf),max(tabPC_QP_pdf)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Gamma(1.5): PC - QP pdf's",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabPC_QP_pdf[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabPC_QP_pdf[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabPC_QP_pdf[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)
graphics.off()

save.image("gamma1dot5GaussMedian.RData")


###  load("gamma1dot5GaussMedian.RData")