###   Generates Figure A3

set.seed(39573)
Nrep = 6
pdf("Gamma5_n250.pdf",width=9,height=5)
cex.label = 1.75
par(mfrow=c(2,3),cex.main=cex.label,cex.axis=cex.label,cex.lab=cex.label,cex.main=cex.label)
###  Gaussian noise
errortype = "normal"
noise.type = "Gaussian"
sdz = 1
c = sdz/sqrt(2)
n = 250
shape = 5
rate = sqrt(shape)
rmodel = function(n) rgamma(n,shape=shape,rate=rate)
dmodel = function(x) dgamma(x,shape=shape,rate=rate)
rmodelz = function(n) rnorm(n,sd=sdz)
unimodal = TRUE
monotone = NULL
K = 200
ylim=c(0,0.55)
xlim = c(-2,7)
lwdT = 6
xxx = seq(0,7,length=200)
plot(xxx,dmodel(xxx),col="red",type="l",main="QP: Gaussian noise",lwd=lwdT,xlab="x",
     ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  QPsupport = c(0,max(y))
  fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=sdz, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE, monotone = monotone, unimod = unimodal, 
                  convex = NULL, support = QPsupport)
  lines(fitQP$x,fitQP$f_X,type="l")
}

plot(xxx,dmodel(xxx),col="red",type="l",main="KD: Gaussian noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitKD = deconvolve(W1 = y,errortype=errortype,sd_U=sdz,
                     xx = seq(min(y), max(y),length.out=K))
  lines(fitKD$x,fitKD$pdf,type="l")
}

plot(xxx,dmodel(xxx),col="red",type="l",main="PC: Gaussian noise",lwd=lwdT,
     xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitPC = deamerKE(y,sigma=sdz,grid.length=K,mu=0,noise.type="Gaussian")
  lines(fitPC$supp,fitPC$f,type="l")
}


###  Laplace noise
errortype = "laplace"
noise.type = "Laplace"
f_Z <- function(z) dlaplace(z,mu=0,b=c)
rmodelz = function(n) rlaplace(n,b=c)

plot(xxx,dmodel(xxx),col="red",type="l",main="QP: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=f_Z, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE, monotone = monotone, unimod = unimodal, 
                  convex = NULL, support = QPsupport)
  lines(fitQP$x,fitQP$f_X,type="l")
}


plot(xxx,dmodel(xxx),col="red",type="l",main="KD: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitKD = deconvolve(W1 = y,errortype=errortype,sd_U=sdz,
                     xx = seq(min(y), max(y),length.out=K))
  lines(fitKD$x,fitKD$pdf,type="l")
}

plot(xxx,dmodel(xxx),col="red",type="l",main="PC: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash")
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitPC = deamerKE(y,sigma=sdz,grid.length=K,mu=0,noise.type="Gaussian")
  lines(fitPC$supp,fitPC$f,type="l")
}

graphics.off()
