###  This is a version of BayesDecon that was modified to allow simulations without plotting

BayesDecon = function (ws, inds, K.t = 10, error.fit = "mixture", variance = "heteroscedastic", 
          burnin, simsize, conf_level = 0.95) 
{
  fn <- function(thetas) {
    vars = B.basis(xs, knots.t) %*% exp(thetas)
    y = -t(thetas) %*% P.t %*% thetas/(2 * s2t) - sum(rep(log(vars), 
                                                          times = mis))/2 - sum(us^2/rep(vars, times = mis))/2
    return(-y)
  }
  gr <- function(thetas) {
    vars = B.basis(xs, knots.t) %*% exp(thetas)
    y = -P.t %*% thetas/s2t
    B.basis.components = matrix(0, nrow = K.t + 1, ncol = n)
    for (kk in 1:(K.t + 1)) {
      thetas.new = rep(0, K.t + 1)
      thetas.new[kk] = 1
      B.basis.components[kk, ] = B.basis(xs, knots.t) %*% 
        thetas.new
    }
    for (kk in 1:(K.t + 1)) for (ii in 1:n) for (jj in (sum(mis[1:ii - 
                                                                1]) + 1):sum(mis[1:ii])) y[kk] = y[kk] - (1 - (us[jj]^2)/vars[ii]) * 
      B.basis.components[kk, ii] * exp(thetas[kk])/(2 * 
                                                      vars[ii])
    return(-y)
  }
  prop.sig.thetas.fn <- function(thetas, s2t) {
    vars = B.basis(xs, knots.t) %*% exp(thetas)
    prop.sig.thetas = matrix(0, nrow = K.t + 1, ncol = K.t + 
                               1)
    B.basis.components = matrix(0, nrow = K.t + 1, ncol = n)
    for (kk in 1:(K.t + 1)) {
      thetas.new = rep(0, K.t + 1)
      thetas.new[kk] = 1
      B.basis.components[kk, ] = B.basis(xs, knots.t) %*% 
        thetas.new
    }
    for (kk in 1:(K.t + 1)) {
      for (ll in kk:(K.t + 1)) {
        if (kk == ll) 
          for (ii in 1:n) for (jj in (sum(mis[1:ii - 
                                              1]) + 1):sum(mis[1:ii])) prop.sig.thetas[kk, 
                                                                                       ll] = prop.sig.thetas[kk, ll] + ((us[jj]^2)/vars[ii] - 
                                                                                                                          1/2) * (B.basis.components[kk, ii] * B.basis.components[ll, 
                                                                                                                                                                                  ii]) * exp(thetas[kk] + thetas[ll])/(vars[ii]^2) + 
              (1 - (us[jj]^2)/vars[ii]) * B.basis.components[kk, 
                                                             ii] * exp(thetas[kk])/(2 * vars[ii])
          else for (ii in 1:n) for (jj in (sum(mis[1:ii - 
                                                   1]) + 1):sum(mis[1:ii])) prop.sig.thetas[kk, 
                                                                                            ll] = prop.sig.thetas[kk, ll] + ((us[jj]^2)/vars[ii] - 
                                                                                                                               1/2) * (B.basis.components[kk, ii] * B.basis.components[ll, 
                                                                                                                                                                                       ii]) * exp(thetas[kk] + thetas[ll])/(vars[ii]^2)
      }
    }
    for (kk in 2:(K.t + 1)) for (ll in 1:(kk - 1)) prop.sig.thetas[kk, 
                                                                   ll] = prop.sig.thetas[ll, kk]
    prop.sig.thetas = prop.sig.thetas + P.t/s2t
    prop.sig.thetas = round(solve(prop.sig.thetas), 4)
    return(prop.sig.thetas)
  }
  P.mat <- function(K) {
    D <- diag(rep(1, K))
    D <- diff(diff(D))
    P <- t(D) %*% D
    return(P)
  }
  B.basis <- function(x, knots) {
    delta <- knots[2] - knots[1]
    n <- length(x)
    K <- length(knots)
    B <- matrix(0, n, K + 1)
    for (jj in 1:(K - 1)) {
      act.inds <- (1:n)[(x >= knots[jj]) & (x <= knots[jj + 
                                                         1])]
      act.x <- x[act.inds]
      resc.x <- (act.x - knots[jj])/(knots[jj + 1] - knots[jj])
      B[act.inds, jj] <- (1/2) * (1 - resc.x)^2
      B[act.inds, jj + 1] <- -(resc.x^2) + resc.x + 1/2
      B[act.inds, jj + 2] <- (resc.x^2)/2
    }
    return(B)
  }
  d.scaled.restricted.mix.norm <- function(x, mean, sd, pi, 
                                           params) {
    if (is.matrix(params)) {
      p = params[, 1]
      mu_curl = params[, 2]
      sigmasq1 = params[, 3]
      sigmasq2 = params[, 4]
    }
    else {
      p = params[1]
      mu_curl = params[2]
      sigmasq1 = params[3]
      sigmasq2 = params[4]
    }
    c1 = (1 - p)/(p^2 + (1 - p)^2)
    c2 = -p/(p^2 + (1 - p)^2)
    mu1 = c1 * mu_curl
    mu2 = c2 * mu_curl
    sd.e = sqrt(var.e.fn(pi, params))
    y = matrix(0, nrow = length(p), ncol = length(x))
    for (kk in 1:length(p)) y[kk, ] = pi[kk] * (p[kk] * dnorm(x, 
                                                              (mean + sd * mu1[kk])/sd.e, sd * sqrt(sigmasq1[kk])/sd.e) + 
                                                  (1 - p[kk]) * dnorm(x, (mean + sd * mu2[kk])/sd.e, 
                                                                      sd * sqrt(sigmasq2[kk])/sd.e))
    y = colSums(y)
    return(y)
  }
  d.restricted.mix.norm <- function(x, mean, sd, params) {
    if (is.matrix(params)) {
      p = params[, 1]
      mu_curl = params[, 2]
      sigmasq1 = params[, 3]
      sigmasq2 = params[, 4]
    }
    else {
      p = params[1]
      mu_curl = params[2]
      sigmasq1 = params[3]
      sigmasq2 = params[4]
    }
    c1 = (1 - p)/(p^2 + (1 - p)^2)
    c2 = -p/(p^2 + (1 - p)^2)
    mu1 = c1 * mu_curl
    mu2 = c2 * mu_curl
    y = p * dnorm(x, mean + sd * mu1, sd * sqrt(sigmasq1)) + 
      (1 - p) * dnorm(x, mean + sd * mu2, sd * sqrt(sigmasq2))
    return(y)
  }
  mixnorm <- function(n, pi, p, mu_curl, sigmasq1, sigmasq2, 
                      e.grid, plot = TRUE) {
    m = length(pi)
    y = numeric(n)
    density <- numeric(length(e.grid))
    inds = sample(1:m, n, TRUE, prob = pi)
    for (ii in 1:m) {
      temp = which(inds == ii)
      y[temp] = r.restricted.mix.norm(length(temp), c(p[ii], 
                                                      mu_curl[ii], sigmasq1[ii], sigmasq2[ii]))
    }
    params = cbind(p, mu_curl, sigmasq1, sigmasq2)
    y = y/sqrt(var.e.fn(pi, params))
    density <- d.scaled.restricted.mix.norm(e.grid, mean = 0, 
                                            sd = 1, pi, params)
    if (plot) {
      hist(y, xlim = c(min(e.grid), max(e.grid)), breaks = 30, 
           freq = FALSE)
      points(e.grid, density, type = "l")
    }
    return(list(es = y, density = density))
  }
  var.e.fn <- function(pi, params) {
    if (is.matrix(params)) {
      p = params[, 1]
      mu_curl = params[, 2]
      sigmasq1 = params[, 3]
      sigmasq2 = params[, 4]
    }
    else {
      p = params[1]
      mu_curl = params[2]
      sigmasq1 = params[3]
      sigmasq2 = params[4]
    }
    c1 = (1 - p)/(p^2 + (1 - p)^2)
    c2 = -p/(p^2 + (1 - p)^2)
    mu1 = c1 * mu_curl
    mu2 = c2 * mu_curl
    y = p * (mu1^{
      2
    } + sigmasq1) + (1 - p) * (mu2^{
      2
    } + sigmasq2)
    y = sum(pi * y)
    return(y)
  }
  r.tnorm.proposal.params.restricted.mix.norm <- function(params) {
    current.p <- params[1]
    current.mu_curl <- params[2]
    current.sigmasq1 <- params[3]
    current.sigmasq2 <- params[4]
    p = rtnorm(1, current.p, 0.01, lower = 0, upper = 1)
    c1 = (1 - p)/(p^2 + (1 - p)^2)
    c2 = -p/(p^2 + (1 - p)^2)
    sigmasq1 = rtnorm(1, current.sigmasq1, 0.1, lower = 0, 
                      upper = Inf)
    sigmasq2 = rtnorm(1, current.sigmasq2, 0.1, lower = 0, 
                      upper = Inf)
    mu_curl = rnorm(1, current.mu_curl, 0.1)
    y = c(p, mu_curl, sigmasq1, sigmasq2)
    return(y)
  }
  e.grid = seq(-4, 4, length.out = 500)
  range_ws = max(ws) - min(ws)
  offset = min(ws)
  ws = (20/range_ws) * (ws - offset)
  mis <- as.data.frame(table(inds))$Freq
  n <- length(unique(inds))
  wbars <- as.vector(tapply(ws, inds, "mean"))
  s2is <- as.vector(tapply(ws, inds, var))
  xs <- current.xs <- start.xs <- as.vector(wbars)
  us <- ws - rep(xs, times = mis)
  x.grid <- seq(min(wbars), max(wbars), length = 500)
  alpha.t <- 0.1
  beta.t <- 0.1
  s2t <- current.s2t <- start.s2t <- 0.1
  P.t <- P.mat(K.t + 1)
  range.start.xs <- diff(range(start.xs))
  knots.t <- seq(min(start.xs) - 0.1 * range.start.xs, max(start.xs) + 
                   0.1 * range.start.xs, length = (K.t))
  optim_results <- optim(rep(1, K.t + 1), fn, gr, method = "BFGS")
  alpha.x = 0.1
  km = 10
  k_init = k.x = 3
  z.x = kmeans(xs, k_init)$cluster
  s.x = rbeta(km, 1, alpha.x)
  pi.x = s.x * c(1, cumprod(1 - s.x[1:(km - 1)]))
  u.x = runif(n, 0, pi.x[z.x])
  xi.x <- 0.9^(1:km - 1)
  mu.x = rep(mean(xs), km)
  for (kk in 1:k_init) {
    mu.x[kk] = mean(xs[z.x == kk])
  }
  sigmasq.x = rep(2 * var(xs)/k_init, km)
  nu0.x = 1/5
  gama0.x = 3
  mu0.x = mean(xs)
  sigmasq0.x = var(xs)/((1 + 1/nu0.x) * (gama0.x/(gama0.x - 
                                                    2)))
  thetas <- current.thetas <- start.thetas <- optim_results$par
  prop.sig.thetas <- prop.sig.thetas.fn(start.thetas, start.s2t)
  if (error.fit == "normal") {
    params.u = matrix(c(0.5, 1, 1, 1), 1, 4)
    z.u = 1
  }
  if (error.fit == "mixture") {
    N = sum(mis)
    a.u <- 1
    b.u <- 1
    sigma0.u <- 3
    alpha.u = 0.2
    km.u = 2
    k_init.u = 2
    ini_kmeans = kmeans(ws, k_init.u)
    z.u = ini_kmeans$cluster
    s_ks.u = rbeta(km.u, 1, alpha.u)
    pi_ks.u = s_ks.u * c(1, cumprod(1 - s_ks.u[1:(km.u - 
                                                    1)]))
    xi.u = 0.9^(1:km.u - 1)
    u_is.u = runif(N, 0, xi.u[z.u])
    params.u = matrix(c(0.5, 0, 1, 1), km.u, 4, byrow = T)
  }
  sig.tune.thetas.1 <- 0
  sig.tune.thetas.2 <- 0.1
  basis.grid = seq(min(knots.t), max(knots.t), by = 0.01)
  B.basis.store = B.basis(basis.grid, knots.t)
  close.ind = rep(0, n)
  var.grid <- seq(min(wbars), max(wbars), length = 100)
  vars <- current.vars <- B.basis(xs, knots.t) %*% exp(current.thetas)
  density.x.est <- numeric(length(x.grid))
  density.x0.est <- matrix(0, length(x.grid), simsize - burnin)
  var.est <- numeric(length(var.grid))
  var.est.store = matrix(0, length(var.grid), simsize - burnin)
  density.e.est <- numeric(length(e.grid))
  density.e0.est <- matrix(0, length(e.grid), simsize - burnin)
  proposed.log.prior <- current.log.prior <- 0
  proposed.log.proposal <- current.log.proposal <- 0
  no.clusters <- numeric(simsize)
  simsize.mh.u = 20
  accepted <- numeric(simsize)
  student.x.grid = (x.grid - mu0.x)/(sqrt(sigmasq0.x) * sqrt(1 + 
                                                               1/nu0.x))
  print("Starting MCMC")
  for (iii in 1:simsize) {
    if (iii%%100 == 0) 
      print(iii)
    u.x = runif(n, min = 0, max = xi.x[z.x])
    for (ii in 1:n) {
      k.x = which(u.x[ii] < xi.x)
      prob.x = pi.x[k.x] * dnorm(xs[ii], mu.x[k.x], sqrt(sigmasq.x[k.x]))/xi.x[k.x]
      z.x[ii] = sample(k.x, 1, TRUE, prob.x[k.x])
    }
    for (kk in 1:km) {
      s.x[kk] = rbeta(1, 1 + sum(z.x == kk), alpha.x + 
                        sum(z.x > kk))
      pi.x = s.x * c(1, cumprod(1 - s.x[1:(km - 1)]))
    }
    k.x = unique(z.x)
    no.clusters[iii] = length(k.x)
    for (kk in k.x) {
      temp = which(z.x == kk)
      xspool = xs[temp]
      nutemp.x = nu0.x + length(xspool)
      mutemp.x = (nu0.x * mu0.x + sum(xspool))/(nutemp.x)
      gamatemp.x = gama0.x + length(xspool)
      sigmasqtemp.x = (gama0.x * sigmasq0.x + sum(xspool^2) + 
                         nu0.x * mu0.x^2 - nutemp.x * mutemp.x^2)/gamatemp.x
      sigmasq.x[kk] = 1/rgamma(1, shape = gamatemp.x/2, 
                               rate = (gamatemp.x * sigmasqtemp.x)/2)
      mu.x[kk] = rnorm(1, mutemp.x, sqrt(sigmasq.x[kk]/nutemp.x))
    }
    for (kk in setdiff(1:km, k.x)) {
      sigmasq.x[kk] = 1/rgamma(1, shape = gama0.x/2, rate = gama0.x * 
                                 sigmasq0.x/2)
      mu.x[kk] = rnorm(1, mu0.x, sqrt(sigmasq.x[kk]/nu0.x))
    }
    if (error.fit == "normal") {
      proposed.xs = rtnorm(n, mean = current.xs, sd = diff(range(start.xs))/6, 
                           lower = min(knots.t), upper = max(knots.t))
      proposed.vars = B.basis(proposed.xs, knots.t) %*% 
        exp(thetas)
      prob.x = (tabulate(z.x) + alpha.x)/(alpha.x + n)
      proposed.prior = current.prior = numeric(n)
      for (kk in k.x) {
        proposed.prior = proposed.prior + prob.x[kk] * 
          dnorm(proposed.xs, mu.x[kk], sqrt(sigmasq.x[kk]))
        current.prior = current.prior + prob.x[kk] * 
          dnorm(current.xs, mu.x[kk], sqrt(sigmasq.x[kk]))
      }
      mh.ratio = proposed.prior/current.prior
      mh.ratio = (mh.ratio * dtnorm(current.xs, mean = proposed.xs, 
                                    sd = diff(range(start.xs))/6, lower = min(knots.t), 
                                    upper = max(knots.t))/dtnorm(proposed.xs, mean = current.xs, 
                                                                 sd = diff(range(start.xs))/6, lower = min(knots.t), 
                                                                 upper = max(knots.t)))
      temp.current.likelihood = dnorm(ws, mean = rep(current.xs, 
                                                     times = mis), sd = rep(sqrt(current.vars), times = mis))
      temp.proposed.likelihood = dnorm(ws, mean = rep(proposed.xs, 
                                                      times = mis), sd = rep(sqrt(proposed.vars), times = mis))
      current.likelihood = tapply(temp.current.likelihood, 
                                  inds, "prod")
      proposed.likelihood = tapply(temp.proposed.likelihood, 
                                   inds, "prod")
      mh.ratio = mh.ratio * proposed.likelihood/current.likelihood
      mh.ratio[is.nan(mh.ratio)] = 0
      u = runif(n)
      inds.to.replace = (1:n)[u < mh.ratio]
      xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
      vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]
      us = ws - rep(xs, times = mis)
      proposed.thetas = rmvnorm(1, current.thetas, diag(rep(sig.tune.thetas.1, 
                                                            (K.t + 1))) + sig.tune.thetas.2 * prop.sig.thetas)
      proposed.thetas = t(proposed.thetas)
      proposed.vars = B.basis(xs, knots.t) %*% exp(proposed.thetas)
      current.log.prior = -t(current.thetas) %*% P.t %*% 
        current.thetas/(2 * s2t)
      proposed.log.prior = -t(proposed.thetas) %*% P.t %*% 
        proposed.thetas/(2 * s2t)
      temp.current.likelihood = dnorm(us, mean = rep(0, 
                                                     times = sum(mis)), sd = rep(sqrt(current.vars), 
                                                                                 times = mis))
      temp.proposed.likelihood = dnorm(us, mean = rep(0, 
                                                      times = sum(mis)), sd = rep(sqrt(proposed.vars), 
                                                                                  times = mis))
      current.log.likelihood = sum(log(temp.current.likelihood))
      proposed.log.likelihood = sum(log(temp.proposed.likelihood))
      log.mh.ratio = proposed.log.prior + proposed.log.likelihood - 
        current.log.likelihood - current.log.prior
      log.u = log(runif(1))
      if (log.u < log.mh.ratio) {
        thetas = current.thetas = proposed.thetas
        vars = current.vars = proposed.vars
      }
    }
    if (error.fit == "mixture") {
      proposed.xs = rtnorm(n, mean = current.xs, sd = diff(range(start.xs))/6, 
                           lower = min(knots.t), upper = max(knots.t))
      for (i in 1:n) {
        close.ind[i] = which.min(abs(proposed.xs[i] - 
                                       basis.grid))
      }
      proposed.vars = B.basis.store[close.ind, ] %*% exp(thetas)
      prob.x = tabulate(z.x)/(n)
      proposed.prior = current.prior = numeric(n)
      for (kk in k.x) {
        proposed.prior = proposed.prior + prob.x[kk] * 
          dnorm(proposed.xs, mu.x[kk], sqrt(sigmasq.x[kk]))
        current.prior = current.prior + prob.x[kk] * 
          dnorm(current.xs, mu.x[kk], sqrt(sigmasq.x[kk]))
      }
      mh.ratio = proposed.prior/current.prior
      mh.ratio = (mh.ratio * dtnorm(current.xs, mean = proposed.xs, 
                                    sd = diff(range(start.xs))/6, lower = min(knots.t), 
                                    upper = max(knots.t))/dtnorm(proposed.xs, mean = current.xs, 
                                                                 sd = diff(range(start.xs))/6, lower = min(knots.t), 
                                                                 upper = max(knots.t)))
      temp.current.likelihood = d.restricted.mix.norm(ws, 
                                                      mean = rep(current.xs, times = mis), sd = rep(sqrt(current.vars), 
                                                                                                    times = mis), params.u[z.u, ])
      temp.proposed.likelihood = d.restricted.mix.norm(ws, 
                                                       mean = rep(proposed.xs, times = mis), sd = rep(sqrt(proposed.vars), 
                                                                                                      times = mis), params.u[z.u, ])
      current.likelihood = tapply(temp.current.likelihood, 
                                  inds, "prod")
      proposed.likelihood = tapply(temp.proposed.likelihood, 
                                   inds, "prod")
      mh.ratio = mh.ratio * proposed.likelihood/current.likelihood
      mh.ratio[is.nan(mh.ratio)] = 0
      u = runif(n)
      inds.to.replace = (1:n)[u < mh.ratio]
      xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
      vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]
      us = ws - rep(xs, times = mis)
    }
    if (variance == "heteroscedastic") {
      proposed.thetas = rep(1, length(thetas))
      for (i in 1:n) {
        close.ind[i] = which.min(abs(xs[i] - basis.grid))
      }
      var.fn = B.basis.store[close.ind, ]
      proposed.vars = var.fn %*% exp(proposed.thetas)
      current.log.prior = -t(current.thetas) %*% P.t %*% 
        current.thetas/(2 * s2t)
      proposed.log.prior = -t(proposed.thetas) %*% P.t %*% 
        proposed.thetas/(2 * s2t)
      temp.current.likelihood = d.restricted.mix.norm(us, 
                                                      mean = rep(0, times = sum(mis)), sd = rep(sqrt(current.vars), 
                                                                                                times = mis), params.u[z.u, ])
      temp.proposed.likelihood = d.restricted.mix.norm(us, 
                                                       mean = rep(0, times = sum(mis)), sd = rep(sqrt(proposed.vars), 
                                                                                                 times = mis), params.u[z.u, ])
      current.log.likelihood = sum(log(temp.current.likelihood))
      proposed.log.likelihood = sum(log(temp.proposed.likelihood))
      log.mh.ratio = proposed.log.prior + proposed.log.likelihood - 
        current.log.likelihood - current.log.prior
      log.u = log(runif(1))
      if (log.u < log.mh.ratio) {
        thetas = current.thetas = proposed.thetas
        vars = current.vars = proposed.vars
      }
    }
    if (variance == "homoscedastic") {
      thetas = rep(1, K.t + 1)
    }
    var.fn.thetas = var.fn %*% exp(thetas)
    var.fn.u = var.fn.thetas[rep(1:n, times = mis), ]
    if (error.fit == "mixture") {
      xs.u = rep(xs, times = mis)
      u_is.u = runif(N, 0, xi.u[z.u])
      for (ii in 1:N) {
        k.ui = which(xi.u > u_is.u[ii])
        c1 = (1 - params.u[k.ui, 1])/(params.u[k.ui, 
                                               1]^2 + (1 - params.u[k.ui, 1])^2)^0.5
        c2 = -params.u[k.ui, 1]/(params.u[k.ui, 1]^2 + 
                                   (1 - params.u[k.ui, 1])^2)^0.5
        mu1 = c1 * params.u[k.ui, 2]
        mu2 = c2 * params.u[k.ui, 2]
        probs.u = (pi_ks.u[k.ui]/xi.u[k.ui]) * (params.u[k.ui, 
                                                         1] * dnorm(ws[ii], xs.u[ii] + sqrt(var.fn.u[ii]) * 
                                                                      mu1, sqrt(var.fn.u[ii] * params.u[k.ui, 3])) + 
                                                  (1 - params.u[k.ui, 1]) * dnorm(ws[ii], xs.u[ii] + 
                                                                                    sqrt(var.fn.u[ii]) * mu2, sqrt(var.fn.u[ii] * 
                                                                                                                     params.u[k.ui, 4])))
        z.u[ii] = sample(k.ui, 1, TRUE, probs.u[k.ui])
      }
      for (kk in 1:km.u) {
        s_ks.u[kk] = rbeta(1, 1 + sum(z.u == kk), alpha.u + 
                             sum(z.u > kk))
      }
      pi_ks.u = s_ks.u * c(1, cumprod(1 - s_ks.u[1:(km.u - 
                                                      1)]))
      k.u = unique(z.u)
      if (iii > 200) 
        simsize.mh.u = 1
      for (rr in 1:simsize.mh.u) {
        for (kk in k.u) {
          temp = which(z.u == kk)
          uspool = us[temp]
          varspool = vars[inds[temp]]
          proposed.params.u <- r.tnorm.proposal.params.restricted.mix.norm(params.u[kk, 
                                                                                    ])
          proposed.log.likelihood = sum(log(d.restricted.mix.norm(uspool, 
                                                                  mean = 0, sd = sqrt(varspool), proposed.params.u)), 
                                        na.rm = T)
          current.log.likelihood = sum(log(d.restricted.mix.norm(uspool, 
                                                                 mean = 0, sd = sqrt(varspool), params.u[kk, 
                                                                                                         ])), na.rm = T)
          log.acc.prob <- proposed.log.likelihood - current.log.likelihood
          if (log(runif(1)) < log.acc.prob) {
            params.u[kk, ] <- proposed.params.u
            accepted[iii] <- 1
          }
        }
      }
      for (j in setdiff(1:km.u, k.u)) {
        params.u[j, 1] = runif(1)
        params.u[j, 2] = rnorm(1, 0, sigma0.u)
        params.u[j, 3:4] = 1/rgamma(2, a.u, 1/b.u)
      }
    }
    s2t = 1/rgamma(1, shape = alpha.t + (K.t + 1)/2, rate = beta.t + 
                     t(thetas) %*% P.t %*% thetas/2)
    if (iii > burnin) {
      prob.x = tabulate(z.x)/(alpha.x + n)
      prob.x[max(k.x) + 1] = alpha.x/(alpha.x + n)
      for (kk in k.x) density.x0.est[, iii - burnin] = density.x0.est[, 
                                                                      iii - burnin] + prob.x[kk] * dnorm(x.grid, mu.x[kk], 
                                                                                                         sqrt(sigmasq.x[kk]))
      density.x0.est[, iii - burnin] = density.x0.est[, 
                                                      iii - burnin] + prob.x[max(k.x) + 1] * dt(student.x.grid, 
                                                                                                df = gama0.x)
      if (error.fit != "mixture") 
        var.est = var.est + B.basis(var.grid, knots.t) %*% 
        exp(thetas)
      var.est.store[, iii - burnin] = B.basis(var.grid, 
                                              knots.t) %*% exp(thetas)
      if (error.fit == "mixture") {
        k.u = unique(z.u)
        prob.e <- tabulate(z.u)/sum(mis)
        var.e = var.e.fn(prob.e[k.u], params.u[k.u, ])
        var.est = var.est + B.basis(var.grid, knots.t) %*% 
          exp(thetas) * var.e
        var.est.store[, iii - burnin] = B.basis(var.grid, 
                                                knots.t) %*% exp(thetas) * var.e
        density.e.est <- density.e.est + d.scaled.restricted.mix.norm(e.grid, 
                                                                      0, 1, prob.e[k.u], params.u[k.u, ])
        density.e0.est[, iii - burnin] = d.scaled.restricted.mix.norm(e.grid, 
                                                                      0, 1, prob.e[k.u], params.u[k.u, ])
      }
    }
  }
  print("MCMC finished")
  if (error.fit == "normal")
    density.e.est <- dnorm(e.grid)
  if (error.fit != "normal")
    density.e.est <- density.e.est/(simsize - burnin)
  x.grid = (range_ws/20) * x.grid + offset
  var.grid = (range_ws/20) * var.grid + offset
  density.x0.est = (20/range_ws) * density.x0.est
  density.x.est <- rowMeans(density.x0.est)
  var.est <- var.est/(simsize - burnin)
  quantiles.density.x = t(apply(t(density.x0.est), 2, sort))
  quantiles.density.e = t(apply(t(density.e0.est), 2, sort))
  quantiles.var = t(apply(t(var.est.store), 2, sort))
  conf_level = 1 - conf_level
  column.index = round(c((simsize - burnin) * conf_level/2,
                         (simsize - burnin) * (1 - conf_level/2)))
  intervals.x = cbind(quantiles.density.x[, column.index[1]],
                      quantiles.density.x[, column.index[2]])
  intervals.e = cbind(quantiles.density.e[, column.index[1]],
                      quantiles.density.e[, column.index[2]])
  intervals.var = cbind(quantiles.var[, column.index[1]], quantiles.var[,
                                                                        column.index[2]])
  def.par <- par(no.readonly = TRUE)
  mat = matrix(c(1, 1, 1, 1, 2, 3), nrow = 3, ncol = 2, byrow = T)
  layout(mat)
  ## readline("Press enter to plot the estimated density of X")
  plot(x.grid, density.x.est, type = "l", main = "Estimated density of X",
       xlab = "X", ylab = "density", ylim = c(0, max(intervals.x[,
                                                                 2])))
  polygon(c(x.grid, rev(x.grid)), c(intervals.x[, 1], rev(intervals.x[,
                                                                      2])), col = "grey75", border = F)
  lines(x.grid, density.x.est, lwd = 2)
  lines(x.grid, intervals.x[, 1], col = "red", lty = 2)
  lines(x.grid, intervals.x[, 2], col = "red", lty = 2)
  ##readline("Press enter to plot the estimated error density")
  plot(e.grid, density.e.est, type = "l", main = "Estimated Error Density ",
       xlab = "error", ylab = "density", ylim = c(0, max(intervals.e[,
                                                                     2])))
  polygon(c(e.grid, rev(e.grid)), c(intervals.e[, 1], rev(intervals.e[,
                                                                      2])), col = "grey75", border = F)
  lines(e.grid, density.e.est, lwd = 2)
  lines(e.grid, intervals.e[, 1], col = "red", lty = 2)
  lines(e.grid, intervals.e[, 2], col = "red", lty = 2)
  ##readline("Press enter to plot the estimated variance function")
  plot(var.grid, var.est, type = "l", main = "Estimated Variance Function",
       xlab = "error", ylab = "value", ylim = c(0, max(intervals.var[,
                                                                     2])))
  polygon(c(var.grid, rev(var.grid)), c(intervals.var[, 1],
                                        rev(intervals.var[, 2])), col = "grey75", border = F)
  lines(var.grid, var.est, lwd = 2)
  lines(var.grid, intervals.var[, 1], col = "red", lty = 2)
  lines(var.grid, intervals.var[, 2], col = "red", lty = 2)
  par(mfrow = c(1, 1))
  result = list(interval = x.grid, density = density.x.est, 
                error_interval = e.grid, error_density = density.e.est, 
                variance_interval = var.grid, variance = var.est, density_post_samples = density.x0.est)
  return(result)
}