###  Generates Figure 10
###  Compares KD and PC to QP using medians

### Normal mixture
cdf2quant = function(xx,cdf,pr)
{
  q = matrix(nrow=length(pr),ncol=1)
  for (i in 1:length(pr))
  {
    ind = (cdf>pr[i])
    q[i] = min(xx[ind])
  }
  q
}
##  Specify the model
library(mixtools)
sdx = sqrt(0.8*0.2*4^2 + 1)
qq = c(0.05,0.1,0.25,0.5,0.75,0.9,0.95)
grid = seq(-1,3,length = 500)*sdx
TRUEcdf = 0.8*pnorm(grid,mean=1/sdx,sd=1/sdx) + 0.2*pnorm(grid,mean=5/sdx,sd=1/sdx)
TRUEquant = cdf2quant(grid,TRUEcdf,qq)
den = "normalMix"
unimodal = FALSE
monotone = NULL
rmodel = function(n) {
  rnormmix(n, lambda=c(0.8,0.2), mu=c(1,5)/sdx, sigma=1/sdx)
}
useLaplace = FALSE
K = 200
variableK = FALSE
##  End model specification

Nrep = 200
t1 = proc.time()
source("runSim.R")
t2 = proc.time()
(t2-t1)/60

widthMult = 0.95
width = 8*widthMult
height = 6*widthMult
cex.label = 1.35

pdf("NormalMixGaussMedian.pdf",width=8,height=6)
par(mfrow=c(2,2))
xlimL = -750
legendCex = 1.1
ylab="log ratio of MedAEs"
plot(sampSize,tabKD_QP_quant[,4],type="b",ylim=c(min(tabKD_QP_quant),
                                                 max(tabKD_QP_quant)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Normal mix: KD - QP quantiles",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabKD_QP_quant[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabKD_QP_quant[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabKD_QP_quant[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## KD versus QP for pdf's
plot(sampSize,tabKD_QP_pdf[,4],type="b",ylim=c(min(tabKD_QP_pdf),max(tabKD_QP_pdf)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Normal mix: KD - QP pdf's",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabKD_QP_pdf[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabKD_QP_pdf[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabKD_QP_pdf[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## PC versus QP for quantiles
plot(sampSize,tabPC_QP_quant[,4],type="b",ylim=c(min(tabPC_QP_quant),
                                                 max(tabPC_QP_quant)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Normal mix: PC - QP quantiles",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabPC_QP_quant[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabPC_QP_quant[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabPC_QP_quant[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)

## PC versus QP for pdf's
plot(sampSize,tabPC_QP_pdf[,4],type="b",ylim=c(min(tabPC_QP_pdf),max(tabPC_QP_pdf)),lty=1,lwd=2,xlab="n",
     ylab=ylab,main="Normal mix: PC - QP pdf's",xlim=c(xlimL,2000),
     cex=cex.label,cex.axis=cex.label,cex.lab=cex.label,xaxt="n")
axis(1, at=c(250,500,1000,2000),cex.axis=cex.label)
lines(sampSize,tabPC_QP_pdf[,3],type="b",col="red",lty=2,lwd=2)
lines(sampSize,tabPC_QP_pdf[,2],type="b",col="blue",lty=3,lwd=2)
lines(sampSize,tabPC_QP_pdf[,1],type="b",col="purple",lty=4,lwd=2)
abline(h=0,lwd=0.5)
legend("topleft",c("0.25","0.5","0.75","1"),lty=c(4,3,2,1),lwd=2,col=c("purple","blue","red","black"),cex=legendCex)
graphics.off()

save.image("normalMixtureGaussMedian.RData")

###  load("normalMixtureGaussMedian.RData")
