###  Generates Figure 8

set.seed(39573)
library(mixtools)
library(deamer)
library(deconvolve)
library(QPdecon)
Nrep = 6
pdf("normalMixture_n250.pdf",width=9,height=5)
cex.label = 1.75
par(mfrow=c(2,3),cex.main=cex.label)
###  Gaussian noise
errortype = "normal"
noise.type = "Gaussian"
sdz = 1
c = sdz/sqrt(2)
sdx = sqrt(0.8*0.2*4^2 + 1)
n = 250
rmodel = function(n) {
  rnormmix(n, lambda=c(0.8,0.2), mu=c(1,5)/sdx, sigma=1/sdx)
}
dmodel = function(x) 0.8*dnorm(x,mean=1/sdx,sd=1/sdx) + 0.2*dnorm(x,mean=5/sdx,sd=1/sdx)
rmodelz = function(n) rnorm(n,sd=sdz)
unimodal = FALSE
monotone = NULL
K = 200
ylim=c(0,0.7)
xlim = c(-1,3)*sdx
xxx = seq(-1,3,length = 200)*sdx
lwdT = 6
plot(xxx,dmodel(xxx),col="red",type="l",main="QP: Gaussian noise",lwd=lwdT,xlab="x",
     ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash",cex.axis=cex.label,cex.lab=cex.label,cex.main=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  QPsupport = range(y)
  fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=sdz, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE, monotone = monotone, unimod = unimodal, 
                  convex = NULL, support = QPsupport)
  lines(fitQP$x,fitQP$f_X,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

plot(xxx,dmodel(xxx),col="red",type="l",main="KD: Gaussian noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="dotted",cex.axis=cex.label,cex.lab=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitKD = deconvolve(W1 = y,errortype=errortype,sd_U=sdz,
                     xx = seq(min(y), max(y),length.out=K))
  lines(fitKD$x,fitKD$pdf,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

plot(xxx,dmodel(xxx),col="red",type="l",main="PC: Gaussian noise",lwd=lwdT,
     xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash",cex.axis=cex.label,cex.lab=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitPC = deamerKE(y,sigma=sdz,grid.length=K,mu=0,noise.type="Gaussian")
  lines(fitPC$supp,fitPC$f,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

###  Laplace noise
errortype = "laplace"
noise.type = "Laplace"
f_Z <- function(z) dlaplace(z,mu=0,b=c)
rmodelz = function(n) rlaplace(n,b=c)

plot(xxx,dmodel(xxx),col="red",type="l",main="QP: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash",cex.axis=cex.label,cex.lab=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=f_Z, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE, monotone = monotone, unimod = unimodal, 
                  convex = NULL, support = QPsupport)
  lines(fitQP$x,fitQP$f_X,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

plot(xxx,dmodel(xxx),col="red",type="l",main="KD: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash",cex.axis=cex.label,cex.lab=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitKD = deconvolve(W1 = y,errortype=errortype,sd_U=sdz,
                     xx = seq(min(y), max(y),length.out=K))
  lines(fitKD$x,fitKD$pdf,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

plot(xxx,dmodel(xxx),col="red",type="l",main="PC: Laplace noise",
     lwd=lwdT,xlab="x",ylab="f(x)",ylim=ylim,xlim=xlim,lty="twodash",cex.axis=cex.label,cex.lab=cex.label,cex.main=cex.label)
for (i in 1:Nrep)
{
  x = rmodel(n)
  z = rmodelz(n)
  y = x + z
  fitPC = deamerKE(y,sigma=sdz,grid.length=K,mu=0,noise.type="Gaussian")
  lines(fitPC$supp,fitPC$f,type="l")
}
lines(xxx,dmodel(xxx),lwd=lwdT,col="red",lty="twodash")

graphics.off()


save.image("normalMixturePlots_n250.RData")

### load("normalMixturePlots_n250.RData")