###  This file is sourced by other files

library(deamer)
library(deconvolve)
library(QPdecon)
cdf2quant = function(xx,cdf,pr)
{
  q = matrix(nrow=length(pr),ncol=1)
  for (i in 1:length(pr))
  {
    ind = (cdf>pr[i])
    q[i] = min(xx[ind])
  }
  q
}
set.seed(47681)
### set.seed(8375)
sampSize = c(250,500,1000,2000)
sdz = (1:4)/4
isim = 0
Nsim = length(sampSize)*length(sdz)*Nrep
MAEresults = matrix(nrow=Nsim,ncol=5)
quantResults = matrix(nrow=Nsim,ncol=3*length(qq))
pdfResults = matrix(nrow=Nsim,ncol=3)
newK = matrix(nrow=Nsim,ncol=1)
lambda = matrix(nrow=Nsim,ncol=1)
for (isize in 1:length(sampSize))
{
  for (isdz in 1:length(sdz))
  {
    
    for (irep in 1:Nrep)
    {
      isim = isim + 1
      MAEresults[isim,1] = sampSize[isize]
      MAEresults[isim,2] =  sdz[isdz]
      x = rmodel(sampSize[isize])
      if (useLaplace == TRUE) 
      {
        c = sdz[isdz]/sqrt(2)
        z = rlaplace(sampSize[isize],b=c) 
        errortype = "laplace"
        noise.type = "Laplace"
        f_Z <- function(z) dlaplace(z,mu=0,b=c)
      }
      if (useLaplace == FALSE) 
      {
        z = rnorm(sampSize[isize],sd=sdz[isdz]) 
        errortype = "normal"
        noise.type = "Gaussian"
        f_Z = sdz[isdz]
      }
      y = x + z
      QPsupport = range(y)
      if (variableK == TRUE) K = min(200,round(3*sqrt(sampSize[isize])))
      if (den == "gamma") QPsupport = c(0,max(y))
      fitQP = QPdecon(y, Pdf = TRUE, K = K, f_Z=f_Z, 
          lambda = "SURE", reg = "2Deriv", integr = TRUE, 
          nonneg = TRUE, monotone = monotone, unimod = unimodal, 
          convex = NULL, support = QPsupport)
      newK[isim] = length(fitQP$x)
      lambda[isim] = fitQP$lambda
      fitKD = deconvolve(W1 = y,errortype=errortype,sd_U=sdz[isdz],
                         xx = seq(min(y), max(y),length.out=newK[isim]))
      fitPC = deamerKE(y,sigma=sdz[isdz],grid.length=newK[isim],mu=0,noise.type=noise.type)
      
      KDcdf = cumsum(fitKD$pdf); KDcdf = KDcdf/KDcdf[length(KDcdf)]
      KDquant = cdf2quant(fitKD$x,KDcdf,qq)  
      MAEresults[isim,3] = mean(abs(KDquant-TRUEquant))
      PCcdf = cumsum(fitPC$f); PCcdf = PCcdf/PCcdf[length(PCcdf)]
      PCquant = cdf2quant(fitPC$supp,PCcdf,qq) 
      MAEresults[isim,4] = mean(abs(PCquant-TRUEquant))
      QPcdf = cumsum(fitQP$f_X); QPcdf = QPcdf/QPcdf[length(QPcdf)]
      QPquant = cdf2quant(fitQP$x,QPcdf,qq)
      MAEresults[isim,5] = mean(abs(QPquant-TRUEquant))
      quantResults[isim,] = c(KDquant,PCquant,QPquant)
      if (den == "normalMix") {truePDF = 0.8*dnorm(fitKD$x,mean=1/sdx,sd=1/sdx) + 0.2*dnorm(fitKD$x,mean=5/sdx,sd=1/sdx)}
      if (den == "gamma") {  truePDF = dgamma(c(0*fitKD$x[fitKD$x < 0],fitKD$x[fitKD$x >= 0]),shape=shape,rate=rate)  }
      if (den == "t") {  truePDF = dt(fitKD$x*sdx,df=df)*sdx  }
      if (den == "norm") { truePDF = dnorm(fitKD$x)}
      delta = fitQP$x[2] -  fitQP$x[1]
      pdfResults[isim,1] = sum(abs(fitKD$pdf - truePDF)) * delta
      pdfResults[isim,2] = sum(abs(fitPC$f - truePDF)) * delta
      pdfResults[isim,3] = sum(abs(fitQP$f_X - truePDF)) * delta
      if (isim/10 == round(isim/10,0)) print(isim)
    }
  }
}
n = MAEresults[,1]
sdError = MAEresults[,2]
l_n = log(n)
l_n = (l_n-mean(l_n))/sd(l_n)
l_n_sq = l_n^2
l_sz = log(sdError)
l_sz = (l_sz-mean(l_sz))/sd(l_sz)
l_sz_sq= l_sz^2
KD_QPquant = log(MAEresults[,3]/MAEresults[,5])  ## KD versus QP
PC_QPquant = log(MAEresults[,4]/MAEresults[,5])  ## PC versus QP

KD_QPpdf = log(pdfResults[,1]/pdfResults[,3])  ## KD versus QP
PC_QPpdf = log(pdfResults[,2]/pdfResults[,3])  ## PC versus QP

tabPC_QP_quant = signif(tapply(PC_QPquant,list(n=n,sd_Error=sdError),median),3)
tabKD_QP_quant = signif(tapply(KD_QPquant,list(n=n,sd_Error=sdError),median),3)

tabPC_QP_pdf = signif(tapply(PC_QPpdf,list(n=n,sd_Error=sdError),median),3)
tabKD_QP_pdf = signif(tapply(KD_QPpdf,list(n=n,sd_Error=sdError),median),3)



