###  Generates Figures 1 and A1

##############################################################
#Code used to analyze the sodium data
##############################################################
#read in sodium data
Y = read.csv("Sodium.csv", header = TRUE)
y.bar = apply(Y, 1, mean) #n-length mean vector
y.sd = apply(Y, 1, sd)  #n-length SD vector
s.pooled = sqrt(mean(y.sd*y.sd)) #pooled sample standard deviation
s.z = s.pooled/sqrt(3) #noise SD, to use in deconvolution below
s.x = sqrt(var(y.bar)-s.z^2/3)
s.z/s.x
pdf("sodiumMeanSD.pdf",width=6,height=5)  ###  Figure A1
plot(y.bar,y.sd,xlab="sample mean",ylab="sample SD")
lines(lowess(y.sd~y.bar),col="red",lwd=4)
legend("topright",c("data","lowess fit"),lty=c(NA,1), pch=c("o",NA), col=c("black","red"), lwd = c("NA",4) )
graphics.off()
s.z/s.x
#Deamer
library(deamer)
fitDeamer = deamerKE(y.bar,sigma=s.z,noise.type = "Gaussian",mu=0)
#Kernel Deconvolution
library(deconvolve)
fitDeconvolve = deconvolve(W1 = y.bar,errortype="normal",sd_U=s.z)
#basic QP deconvolution with only integrate-to-one and nonnegativity
library(QPdecon)
QP.Na.basic = QPdecon(y.bar, Pdf = TRUE, K = NULL, f_Z=s.z, 
                lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                nonneg = TRUE, monotone = NULL, unimod = FALSE, convex = NULL, support = NULL)
#QP deconvolution with additional support constraint
QP.Na.support = QPdecon(y.bar, Pdf = TRUE, K = NULL, f_Z=s.z, lambda = "SURE", 
                reg = "2Deriv", integr = TRUE, nonneg = TRUE, 
                monotone = NULL, unimod = FALSE, convex = NULL, support = c(0,Inf))
#QP deconvolution with additional support and unimodality constraints that works 
QP.Na.support.uni = QPdecon(y.bar, Pdf = TRUE, K = NULL, f_Z=s.z, lambda = "SURE", 
                reg = "2Deriv", integr = TRUE, nonneg = TRUE, monotone = NULL, 
                unimod = TRUE, convex = NULL, support = c(0,1.5*max(y.bar)))
pdf("sodium01.pdf",width=6,height=4)  ###  Figure 1
N = length(y.bar)
K = min(200, floor(3 * sqrt(N)))
hist(y.bar,K,freq=FALSE,ylim=c(0,0.2),xlab="Sodium concentration (ppm)",main="",ylab="density")
### lines(fitDeconvolve$x,fitDeconvolve$pdf,lwd=4,lty="solid")
lines(QP.Na.support$x,QP.Na.support$f_X,col="black",lwd=4,lty="longdash")
lines(fitDeamer$supp,fitDeamer$f,lty="solid",lwd=4,col="red")
legend("topright",c("QP","PC"),
       col=c("black","red"),lty=c("solid","longdash"),lwd=4)
graphics.off()

#QP deconvolution with additional support and unimodality constraints that works 
QP.Na = QPdecon(y.bar, Pdf = TRUE, K = NULL, f_Z=s.z, lambda = "SURE", 
                reg = "2Deriv", integr = TRUE, nonneg = TRUE, monotone = NULL, 
                unimod = TRUE, convex = NULL, support = c(0,1.5*max(y.bar)))
plot(QP.Na, histo=TRUE)
QP.Na$lambda

#QP deconvolution with additional support and unimodality constraints that gives an error 
QP.Na = QPdecon(y.bar, Pdf = TRUE, K = NULL, f_Z=s.z, lambda = "SURE", 
                reg = "2Deriv", integr = TRUE, nonneg = TRUE, 
                monotone = NULL, unimod = TRUE, convex = NULL, support = c(0,1.5*max(y.bar)))
plot(QP.Na, histo=TRUE)

#Try the scree plot to see if the SURE-selected parameter was OK
ScreePlot(y.bar, Pdf = TRUE, K = NULL, f_Z = s.z, lambda = "Scree", reg = "2Deriv", 
          integr = TRUE, nonneg = TRUE, monotone = NULL, unimod = FALSE, 
          convex = NULL, support = NULL)
